@@ -23,6 +23,8 @@ from cubicweb.selectors import is_instance
 
 from cubes.tracker import hooks as tracker
 
+# search for ticket closing instructions #######################################
+
 TICKET_EID_RGX = re.compile('#(\d+)')
 
 class SearchRevisionTicketOp(hook.DataOperationMixIn,
@@ -68,6 +70,27 @@ class RevisionAdded(hook.Hook):
             SearchRevisionTicketOp.get_instance(self._cw).add_data(self.entity)
 
 
+# synchronize  "P source_repository R" with ####################################
+# "P has_apycot_environment PE AND PE local_repository R"
+
+class SetSourceRepositoryOp(hook.Operation):
+    def precommit_event(self):
+        if not self.projet.source_repository and self.projectenv.repository:
+            self.projet.set_relations(source_repository=self.projectenv.repository)
+
+class ApycotEnvironmentAdded(hook.Hook):
+    __regid__ = 'trackervcs.revistion-to-ticket-state'
+    __select__ = hook.Hook.__select__ & hook.match_rtype('has_apycot_environment')
+    events = ('after_add_relation',)
+
+    def __call__(self):
+        project = self._cw.entity_from_eid(self.eidfrom)
+        if not project.source_repository:
+            projectenv = self._cw.entity_from_eid(self.eidto)
+            SetSourceRepositoryOp(self._cw, project=project, projectenv=projectenv)
+
+# security propagation #########################################################
+
 tracker.S_RELS |= set(('source_repository', 'has_patch_repository',
                        'patch_revision',
                        'has_activity', ))
@@ -77,3 +100,4 @@ tracker.O_RELS |= set(('patch_repository', 'point_of',))
 def registration_callback(vreg):
     if vreg.config['trusted-vcs-repositories']:
         vreg.register(RevisionAdded)
+
